@@ -45,6 +45,7 @@ NFINDREPLACE g_find;
 TCHAR g_findbuf[128];
 extern NTOOLBAR g_toolbar;
 void* g_ModalDlgData;
+BYTE g_MiniCommand = false;
 
 NSIS_ENTRYPOINT_SIMPLEGUI
 int WINAPI _tWinMain(HINSTANCE hInst,HINSTANCE hOldInst,LPTSTR CmdLineParams,int ShowCmd) {
@@ -55,10 +56,9 @@ int WINAPI _tWinMain(HINSTANCE hInst,HINSTANCE hOldInst,LPTSTR CmdLineParams,int
   if (SDDA) ((BOOL(WINAPI*)(LPCSTR))SDDA)(""); // Remove the current directory from the default DLL search order
 
   // Try to register the SysLink class
-  DWORD iccestruct[2] = { 8, 0x8000 }; // ICC_LINK_CLASS (ComCtl32v6)
-  FARPROC icce = SupportsW95() ? GetSysProcAddr("COMCTL32", "InitCommonControlsEx") : (FARPROC) InitCommonControlsEx;
-  BOOL succ = ((BOOL(WINAPI*)(const void*))icce)(iccestruct);
-  if (!succ && (sizeof(void*) > 4 || LOBYTE(GetVersion()) >= 5))
+  enum { icc_link_class = 0x8000 }; // ComCtl32v6
+  #if (!defined(_MSC_VER) && !defined(_WIN64)) || (defined(_MSC_VER) && (defined(_M_IX86) || defined(_M_IA64))) // x86 or Itanium
+  if (!InitCCEx(icc_link_class) && (sizeof(void*) > 4 || LOBYTE(GetVersion()) >= 5)) // Must check the version because older shell32 versions have a incompatible function at the same ordinal
   {
     FARPROC lwrc = GetSysProcAddr("SHELL32", (LPCSTR) 258); // LinkWindow_RegisterClass
     if (lwrc) ((BOOL(WINAPI*)())lwrc)();
@@ -69,6 +69,7 @@ int WINAPI _tWinMain(HINSTANCE hInst,HINSTANCE hOldInst,LPTSTR CmdLineParams,int
       RegisterClass(&wc); // Superclass the old link window class if SysLink is not available
     }
   }
+  #endif
 
   memset(&g_sdata,0,sizeof(NSCRIPTDATA));
   memset(&g_resize,0,sizeof(NRESIZEDATA));
@@ -78,6 +79,7 @@ int WINAPI _tWinMain(HINSTANCE hInst,HINSTANCE hOldInst,LPTSTR CmdLineParams,int
   g_sdata.sigint_event_legacy = CreateEvent(NULL, FALSE, FALSE, MakensisAPI::SigintEventNameLegacy);
   g_sdata.verbosity = (unsigned char) ReadRegSettingDW(REGVERBOSITY, 4);
   if (g_sdata.verbosity > 4) g_sdata.verbosity = 4;
+  g_sdata.log_zoom = 100;
   RestoreSymbols();
   LoadSysLibrary("RichEd20");
 
@@ -86,12 +88,12 @@ int WINAPI _tWinMain(HINSTANCE hInst,HINSTANCE hOldInst,LPTSTR CmdLineParams,int
     return 1;
   }
   ResetObjects();
+  HACCEL haccel = LoadAccelerators(g_sdata.hInstance, MAKEINTRESOURCE(IDK_ACCEL));
   HWND hDialog = CreateDialog(g_sdata.hInstance,MAKEINTRESOURCE(DLG_MAIN),0,DialogProc);
-  if (!hDialog) {
+  if (!hDialog && !g_MiniCommand) {
     MessageBox(0,DLGERROR,ERRBOXTITLE,MB_ICONEXCLAMATION|MB_OK|MB_TASKMODAL);
     return 1;
   }
-  HACCEL haccel = LoadAccelerators(g_sdata.hInstance, MAKEINTRESOURCE(IDK_ACCEL));
   MSG  msg;
   int status;
   while ((status=GetMessage(&msg,0,0,0))!=0) {
@@ -146,22 +148,21 @@ static void AddScriptCmdArgs(const TCHAR *arg)
   GlobalUnlock(g_sdata.script_cmd_args);
 }
 
-static void ProcessCommandLine()
+enum { CMD_PICKCOMP = 0x0001, CMD_SPY = 0x0080, CMD_LOOKUP = 0x8000 };
+static UINT ProcessCommandLine()
 {
   TCHAR **argv;
-  int i, j;
+  int i, j, retflags = 0;
   int argc = SetArgv((TCHAR *)GetCommandLine(), &argv);
   if (argc > 1) {
     for (i = 1; i < argc; i++)
     {
-      if (!StrCmpNI(argv[i], _T("/XSetCompressor "), lstrlen(_T("/XSetCompressor "))))
+      if (!lstrcmpi(argv[i], _T("/Spy"))) retflags |= CMD_SPY;
+      else if (!lstrcmpi(argv[i], _T("/Lookup"))) retflags |= CMD_LOOKUP;
+      else if (!StrCmpNI(argv[i], _T("/XSetCompressor "), COUNTOF("/XSetCompressor ") - !0))
       {
-        TCHAR *p = argv[i] + lstrlen(_T("/XSetCompressor "));
-        if(!StrCmpNI(p,_T("/FINAL "), lstrlen(_T("/FINAL "))))
-        {
-          p += lstrlen(_T("/FINAL "));
-        }
-
+        TCHAR *p = argv[i] + lstrlen(_T("/XSetCompressor ")), cchSlashFinalSpace = COUNTOF("/FINAL ") - !0;
+        if (!StrCmpNI(p,_T("/FINAL "), cchSlashFinalSpace)) p += cchSlashFinalSpace;
         while (*p == _T(' ')) p++;
 
         for (j = (int) COMPRESSOR_SCRIPT + 1; j < (int) COMPRESSOR_BEST; j++)
@@ -174,7 +175,7 @@ static void ProcessCommandLine()
       }
       else if (!lstrcmpi(argv[i], _T("/ChooseCompressor")))
       {
-        g_sdata.userSelectCompressor = TRUE;
+        retflags |= CMD_PICKCOMP;
       }
       else if (argv[i][0] == _T('-') || argv[i][0] == _T('/'))
       {
@@ -189,6 +190,7 @@ static void ProcessCommandLine()
     }
   }
   MemSafeFree(argv);
+  return retflags;
 }
 
 DWORD CALLBACK SaveFileStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
@@ -209,7 +211,7 @@ DWORD CALLBACK SaveFileStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb
 static void ToolBarSizeChanged(HWND hDlg)
 {
   RECT r;
-  HWND hEd = GetDlgItem(hDlg, IDC_LOGWIN);
+  HWND hEd = g_sdata.logwnd;
   GetWindowRect(g_toolbar.hwnd, &r);
   LONG tbh = RectH(r);
   GetWindowRect(hEd, &r);
@@ -255,37 +257,59 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
   switch (msg) {
     case WM_INITDIALOG:
     {
-      g_sdata.hwnd=hwndDlg;
+      g_sdata.hwnd=hwndDlg, g_sdata.logwnd = GetDlgItem(hwndDlg, IDC_LOGWIN);
       HICON hIcon = LoadIcon(g_sdata.hInstance,MAKEINTRESOURCE(IDI_ICON));
       SetClassLongPtr(hwndDlg,GCLP_HICON,(LONG_PTR)hIcon);
       // Altered by Darren Owen (DrO) on 29/9/2003
       // Added in receiving of mouse and key events from the richedit control
-      SendDlgItemMessage(hwndDlg,IDC_LOGWIN,EM_SETEVENTMASK,(WPARAM)NULL,ENM_SELCHANGE|ENM_MOUSEEVENTS|ENM_KEYEVENTS);
-      DragAcceptFiles(g_sdata.hwnd,FALSE);
+      SendMessage(g_sdata.logwnd,EM_SETEVENTMASK,(WPARAM)NULL,ENM_SELCHANGE|ENM_MOUSEEVENTS|ENM_KEYEVENTS);
+      InitializeLogWindow();
       g_sdata.menu = GetMenu(g_sdata.hwnd);
       g_sdata.fileSubmenu = FindSubMenu(g_sdata.menu, IDM_FILE);
       g_sdata.editSubmenu = FindSubMenu(g_sdata.menu, IDM_EDIT);
       g_sdata.toolsSubmenu = FindSubMenu(g_sdata.menu, IDM_TOOLS);
+      SetMenuDefaultItem(FindSubMenu(g_sdata.menu, IDM_SCRIPT), IDM_RECOMPILE_TEST, MF_BYCOMMAND);
       RestoreMRUList();
       CreateToolBar();
       InitTooltips(g_sdata.hwnd);
       SetDlgItemText(g_sdata.hwnd,IDC_VERSION,g_sdata.branding);
-      HFONT hFont = CreateFontPt(hwndDlg,8,FW_NORMAL,FIXED_PITCH|FF_DONTCARE,DEFAULT_CHARSET,_T("Courier New"));
-      SendDlgItemMessage(hwndDlg,IDC_LOGWIN,WM_SETFONT,(WPARAM)hFont,0);
-      RestoreWindowPos(g_sdata.hwnd);
-      RestoreCompressor();
-      SetScript(_T(""));
+      LPCTSTR fontname = _T("Courier New"), fontconsolas = _T("Consolas");
+      BYTE fontsize = 8, fontcharset = DEFAULT_CHARSET, suppwin4 = SupportsWNT4() || SupportsW9X();
+      if (FontExists(fontconsolas))
+      {
+        fontname = fontconsolas, ++fontsize;
+      }
+      else if (SupportsW2000() && GetACP() == 932) // According to older Inno, Courier New cannot display Japanese on < WinXP
+      {
+        LPCWSTR msgothlocalutf = L"\xff2d\xff33 \xff30\x30b4\x30b7\x30c3\x30af";
+        const CHAR msgothlocal932[] = { -126, 'l', -126, 'r', ' ', -125, 'S', -125, 'V', -125, 'b', -125, 'N', '\0' };
+        fontcharset = SHIFTJIS_CHARSET, ++fontsize;
+        fontname = _T("MS Gothic"); // Win2000 can handle this, downlevel cannot
+        if (suppwin4 && !FontExists(fontname)) fontname = sizeof(TCHAR) > 1 ? (LPCTSTR) msgothlocalutf : (LPCTSTR) msgothlocal932;
+      }
+      HFONT hFont = CreateFontPt(hwndDlg,fontsize,FW_NORMAL,FIXED_PITCH|FF_DONTCARE,fontcharset,fontname);
+      SendMessage(g_sdata.logwnd,WM_SETFONT,(WPARAM)hFont,0);
       g_sdata.compressor = COMPRESSOR_NONE_SELECTED;
-      g_sdata.userSelectCompressor = FALSE;
+      SetScript(_T(""));
+      RestoreCompressor();
       ToolBarSizeChanged(hwndDlg);
 
-      ProcessCommandLine();
+      UINT docmd = ProcessCommandLine();
+      if ((docmd & (CMD_SPY|CMD_LOOKUP)))
+      {
+        INT_PTR r = ((docmd & CMD_LOOKUP) ? ShowLookupDialog : ShowWndSpy)(0);
+        g_sdata.hwnd = NULL; // Don't save window pos
+        g_MiniCommand++;
+        return SendMessage(hwndDlg, WM_CLOSE, r, r);
+      }
+
+      RestoreWindowPos(g_sdata.hwnd);
 
       if(g_sdata.compressor == COMPRESSOR_NONE_SELECTED) {
         SetCompressor(g_sdata.default_compressor);
       }
 
-      if(g_sdata.userSelectCompressor) {
+      if(docmd & CMD_PICKCOMP) {
         if (DialogBox(g_sdata.hInstance,MAKEINTRESOURCE(DLG_COMPRESSOR),g_sdata.hwnd,API_cast<DLGPROC>(CompressorProc))) {
           EnableItems(g_sdata.hwnd);
           return TRUE;
@@ -306,10 +330,10 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
     }
     case WM_DESTROY:
     {
-      DragAcceptFiles(g_sdata.hwnd, FALSE);
       SaveSymbols();
       SaveMRUList();
       SaveWindowPos(g_sdata.hwnd);
+      ReleaseLogWindow();
       ImageList_Destroy(g_toolbar.imagelist);
       ImageList_Destroy(g_toolbar.imagelistd);
       ImageList_Destroy(g_toolbar.imagelisth);
@@ -317,10 +341,11 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       PostQuitMessage(0);
       return TRUE;
     }
-    case WM_CLOSE:
+    case WM_CLOSE: tryquitapp:
     {
       if (!g_sdata.thread) {
         DestroyWindow(hwndDlg);
+        PostQuitMessage((int) wParam);
       }
       return TRUE;
     }
@@ -474,7 +499,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           SetLogColor(LC_WARNING);
         }
         else {
-          SetTitle(g_sdata.hwnd,_T("Finished Sucessfully"));
+          SetTitle(g_sdata.hwnd,_T("Finished Successfully"));
           PlayAppSoundAsync(("BuildComplete"), MB_ICONASTERISK);
           SetLogColor(LC_SUCCESS);
         }
@@ -497,15 +522,16 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
     case MakensisAPI::QUERYHOST: {
       if (MakensisAPI::QH_OUTPUTCHARSET == wParam) {
         const UINT reqcp = 1200; // We want UTF-16LE
-        SetWindowLongPtr(hwndDlg, DWLP_MSGRESULT, (LONG_PTR)(1+reqcp));
-        return TRUE;
+        return DlgRet(hwndDlg, (LONG_PTR)(1+reqcp));
       }
+      else if (MakensisAPI::QH_SUPPORTEDVERSION == wParam)
+        return DlgRet(hwndDlg, 0x03006000);
       return FALSE;
     }
     case WM_NOTIFY:
       switch (((NMHDR*)lParam)->code ) {
         case EN_SELCHANGE:
-          EnableMenuItem(g_sdata.menu, IDM_COPYSELECTED, RicheditHasSelection(GetDlgItem(hwndDlg, IDC_LOGWIN)) ? MF_ENABLED : MF_GRAYED);
+          EnableMenuItem(g_sdata.menu, IDM_COPYSELECTED, RicheditHasSelection(g_sdata.logwnd) ? MF_ENABLED : MF_GRAYED);
           break;
         
         // Altered by Darren Owen (DrO) on 6/10/2003
@@ -518,7 +544,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           if(WM_RBUTTONUP == lpnmMsg->msg || (WM_KEYUP == lpnmMsg->msg && lpnmMsg->wParam == VK_APPS))
           {
             POINT pt;
-            HWND edit = GetDlgItem(g_sdata.hwnd,IDC_LOGWIN);
+            HWND edit = g_sdata.logwnd;
             RECT r;
             GetCursorPos(&pt);
 
@@ -530,7 +556,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
             GetClientRect(edit, &r);
             if (!PtInRect(&r, pt)) pt.x = pt.y = 0;
             MapWindowPoints(edit, HWND_DESKTOP, &pt, 1);
-            TrackPopupMenu(g_sdata.editSubmenu, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, pt.x, pt.y, 0, g_sdata.hwnd, 0);
+            TrackPopupMenu(g_sdata.editSubmenu, GetMenuDropAlignment() | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, pt.x, pt.y, 0, g_sdata.hwnd, 0);
           }
           break;
         case TBN_DROPDOWN:
@@ -546,7 +572,8 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       return TRUE;
     case WM_COPYDATA:
     {
-      PCOPYDATASTRUCT cds = PCOPYDATASTRUCT(lParam);
+      using namespace MakensisAPI;
+      COPYDATASTRUCT *cds = (COPYDATASTRUCT*) lParam, cdsret;
       switch (cds->dwData) {
         case MakensisAPI::NOTIFY_SCRIPT:
           MemSafeFree(g_sdata.input_script);
@@ -564,6 +591,24 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           g_sdata.output_exe = (TCHAR*) MemAlloc(cds->cbData * sizeof(TCHAR));
           lstrcpy(g_sdata.output_exe, (TCHAR *)cds->lpData);
           break;
+        case MakensisAPI::PROMPT_FILEPATH:
+          if ((((PROMPT_FILEPATH_DATA*)cds->lpData)->Platform & 7) == sizeof(TCHAR))
+          {
+            TCHAR buf[MAX_PATH];
+            lstrcpyn(buf, FSPath::FindLastComponent(((PROMPT_FILEPATH_DATA*)cds->lpData)->Path), COUNTOF(buf));
+            OPENFILENAME of = { SizeOfStruct(of) };
+            of.hwndOwner = hwndDlg;
+            of.lpstrFilter = _T("*.exe\0*.exe\0*\0*.*\0");
+            of.lpstrFile = buf, of.nMaxFile = COUNTOF(buf);
+            of.Flags = OFN_EXPLORER|OFN_ENABLESIZING|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST|OFN_NOCHANGEDIR;
+            if (GetSaveFileName(&of))
+            {
+              cdsret.dwData = cds->dwData, cdsret.cbData = (lstrlen(buf) + 1) * sizeof(TCHAR), cdsret.lpData = buf;
+              SendMessage((HWND) wParam, WM_COPYDATA, (SIZE_T) hwndDlg, (SIZE_T) &cdsret);
+            }
+            return TRUE;
+          }
+          return FALSE;
       }
       return TRUE;
     }
@@ -572,12 +617,27 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       break;
     case WM_MAKENSIS_UPDATEUISTATE:
     {
-      UINT i, emptylog = SendDlgItemMessage(hwndDlg, IDC_LOGWIN, WM_GETTEXTLENGTH, 0, 0) == 0;
+      UINT i, emptylog = SendMessage(g_sdata.logwnd, WM_GETTEXTLENGTH, 0, 0) == 0;
       static const PACKEDCMDID_T nonemptylogids [] = { PACKCMDID(IDM_COPY), PACKCMDID(IDM_COPYALL), PACKCMDID(IDM_CLEARLOG), PACKCMDID(IDM_SELECTALL) };
       for (i = 0; i < COUNTOF(nonemptylogids); ++i) EnableUICommand(UNPACKCMDID(nonemptylogids[i]), !emptylog);
       EnableUICommand(IDM_BROWSESCR, !!g_sdata.input_script);
       break;
     }
+    case WM_MAKENSIS_FREEZEEDITOR:
+      RicheditFreeze(g_sdata.pLogTextDoc, lParam);
+      break;
+    case WM_TIMER:
+    {
+      HWND hCtl;
+      switch(wParam) {
+        case TID_CONFIGURECLOSEORABORT:
+          SendMessage(hCtl = GetDlgItem(hwndDlg, IDCANCEL), WM_SETTEXT, 0, (LPARAM) (g_sdata.thread ? _T("&Abort") : _T("&Close")));
+          EnableWindow(hCtl, true);
+          UpdateCloseButtonTooltip();
+          return KillTimer(hwndDlg, wParam);
+      }
+      break;
+    }
     case WM_COMMAND:
     {
       switch (LOWORD(wParam)) {
@@ -590,7 +650,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
             lstrcpy(str,g_sdata.input_script);
             str2=_tcsrchr(str,_T('\\'));
             if(str2!=NULL) *(str2+1)=0;
-            ShellExecute(g_sdata.hwnd,_T("open"),str,NULL,NULL,SW_SHOWNORMAL);
+            ShellExecuteWithErrorBox(hwndDlg, str);
           }
           return TRUE;
         }
@@ -601,13 +661,13 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
         case IDM_ABOUT: return ShowAboutDialog(hwndDlg)|TRUE;
         case IDM_SELECTALL:
         {
-          SendDlgItemMessage(g_sdata.hwnd, IDC_LOGWIN, EM_SETSEL, 0, -1);
+          SendMessage(g_sdata.logwnd, EM_SETSEL, 0, -1);
           return TRUE;
         }
         case IDM_LOADSCRIPT:
         {
           if (!g_sdata.thread) {
-            OPENFILENAME l={sizeof(l),};
+            OPENFILENAME l = { SizeOfStruct(l) };
             TCHAR buf[MAX_PATH];
             l.hwndOwner = hwndDlg;
             l.lpstrFilter = _T("NSIS Script (*.nsi)\0*.nsi\0All Files (*.*)\0*.*\0");
@@ -644,10 +704,17 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
         case IDM_CLEARLOG:
         {
           if (!g_sdata.thread) {
-            ClearLog(g_sdata.hwnd);
+            ClearLog();
           }
           return TRUE;
         }
+        case IDM_ZOOM_INC: g_sdata.log_zoom += 25; goto set_log_zoom;
+        case IDM_ZOOM_DEC: g_sdata.log_zoom -= 25; goto set_log_zoom;
+        case IDM_ZOOM_RST:
+          g_sdata.log_zoom = 100; set_log_zoom:
+          SendMessage(g_sdata.logwnd, EM_SETZOOM, g_sdata.log_zoom = STD_MAX((int)g_sdata.log_zoom, 25), 100);
+          InvalidateRect(g_sdata.logwnd, 0, false);
+          break;
         case IDM_RECOMPILE:
         {
           CompileNSISScript();
@@ -665,42 +732,47 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           DialogBox(g_sdata.hInstance,MAKEINTRESOURCE(DLG_SETTINGS),g_sdata.hwnd,API_cast<DLGPROC>(SettingsProc));
           return TRUE;
         }
-        case IDM_WNDSPY:
+        case IDM_WNDSPY: return ShowWndSpy(hwndDlg);
+        case IDM_LOOKUP: return ShowLookupDialog(hwndDlg);
+        case IDM_GUIDGEN:
         {
-          extern int ShowWndSpy(HWND hOwner);
-          return ShowWndSpy(g_sdata.hwnd);
+          GUID guid;
+          TCHAR buf[41 * (1 + (sizeof(TCHAR) < 2))];
+          FARPROC func = GetKeyState(VK_CONTROL) < 0 ? GetSysProcAddr("RPCRT4", "UuidCreateSequential") : NULL;
+          ((HRESULT(WINAPI*)(GUID*))(func ? func : GetSysProcAddr("RPCRT4", "UuidCreate")))(&guid);
+          ((int(WINAPI*)(GUID*, TCHAR*, int))(GetSysProcAddr("OLE32", "StringFromGUID2")))(&guid, buf, 39);
+          for (UINT i = 0; sizeof(TCHAR) < 2; ++i) if (!(buf[i] = (CHAR) ((WCHAR*)buf)[i])) break; // WCHAR to TCHAR if ANSI
+          LogMessage(g_sdata.hwnd, (buf[38] = '\r', buf[39] = '\n', buf[40] = '\0', buf));
+          SendMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0, 0); // Update clear log command state
+          break;
         }
         case IDM_TEST:
         case IDC_TEST:
         {
           if (g_sdata.output_exe) {
-            ShellExecute(g_sdata.hwnd,_T("open"),g_sdata.output_exe,NULL,NULL,SW_SHOWNORMAL);
+            ShellExecuteWithErrorBox(hwndDlg, g_sdata.output_exe);
           }
           return TRUE;
         }
         case IDM_EDITSCRIPT:
         {
           if (g_sdata.input_script) {
-            LPCTSTR verb = _T("open"); // BUGBUG: Should not force the open verb?
-            HINSTANCE hi = ShellExecute(g_sdata.hwnd,verb,g_sdata.input_script,NULL,NULL,SW_SHOWNORMAL);
-            if ((UINT_PTR)hi <= 32) {
-              TCHAR path[MAX_PATH];
-              if (GetWindowsDirectory(path,sizeof(path))) {
-                lstrcat(path,_T("\\notepad.exe"));
-                ShellExecute(g_sdata.hwnd,verb,path,g_sdata.input_script,NULL,SW_SHOWNORMAL);
+            if (!ShellExecuteSilent(hwndDlg, g_sdata.input_script)) {
+              TCHAR app[MAX_PATH];
+              if (GetWindowsDirectory(app, COUNTOF(app))) {
+                lstrcat(app,_T("\\notepad.exe"));
+                ShellExecuteWithErrorBox(hwndDlg, app, g_sdata.input_script);
               }
             }
           }
           return TRUE;
         }
         case IDCANCEL:
+          if (g_sdata.thread)
+            return PostMessage(hwndDlg, WM_COMMAND, IDM_CANCEL, 0);
         case IDM_EXIT:
-        {
-          if (!g_sdata.thread) {
-            DestroyWindow(g_sdata.hwnd);
-          }
-          return TRUE;
-        }
+          wParam = 0;
+          goto tryquitapp;
         case IDM_CANCEL:
         {
           SetEvent(g_sdata.sigint_event);
@@ -708,17 +780,17 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           return TRUE;
         }
         case IDM_COPY:
-          if (RicheditHasSelection(GetDlgItem(hwndDlg, IDC_LOGWIN))) goto logwndcopysel;
+          if (RicheditHasSelection(g_sdata.logwnd)) goto logwndcopysel;
           // fall through
         case IDM_COPYALL:
           CopyToClipboard(g_sdata.hwnd);
           return TRUE;
         case IDM_COPYSELECTED: logwndcopysel:
-          SendDlgItemMessage(g_sdata.hwnd, IDC_LOGWIN, WM_COPY, 0, 0);
+          SendMessage(g_sdata.logwnd, WM_COPY, 0, 0);
           return TRUE;
         case IDM_SAVE:
         {
-          OPENFILENAME l={sizeof(l),};
+          OPENFILENAME l = { SizeOfStruct(l) };
           TCHAR buf[MAX_STRING];
           l.hwndOwner = hwndDlg;
           l.lpstrFilter = _T("Log Files (*.log)\0*.log\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0");
@@ -735,7 +807,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
               WPARAM opts = sizeof(TCHAR) > 1 ? (SF_TEXT|SF_UNICODE) : (SF_TEXT);
               DWORD_PTR cookie[2] = { (DWORD_PTR)hFile, FALSE };
               EDITSTREAM es = { (DWORD_PTR)&cookie, 0, SaveFileStreamCallback };
-              SendMessage(GetDlgItem(g_sdata.hwnd, IDC_LOGWIN), EM_STREAMOUT, opts, (LPARAM)&es);
+              SendMessage(g_sdata.logwnd, EM_STREAMOUT, opts, (LPARAM)&es);
               CloseHandle(hFile);
             }
           }
@@ -776,13 +848,13 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       if (lpfr->Flags & FR_MATCHCASE) flags |= FR_MATCHCASE;
       if (lpfr->Flags & FR_WHOLEWORD) flags |= FR_WHOLEWORD;
       FINDTEXTEX ft;
-      SendDlgItemMessage(hwndDlg, IDC_LOGWIN, EM_EXGETSEL, 0, (LPARAM)&ft.chrg);
+      SendMessage(g_sdata.logwnd, EM_EXGETSEL, 0, (LPARAM)&ft.chrg);
       ft.chrg.cpMin = (ft.chrg.cpMax == ft.chrg.cpMin) ? 0 : ft.chrg.cpMax;
-      ft.chrg.cpMax = (LONG) SendDlgItemMessage(hwndDlg, IDC_LOGWIN, WM_GETTEXTLENGTH, 0, 0);
+      ft.chrg.cpMax = (LONG) SendMessage(g_sdata.logwnd, WM_GETTEXTLENGTH, 0, 0);
       ft.lpstrText = lpfr->lpstrFindWhat;
-      ft.chrg.cpMin = (LONG) SendDlgItemMessage(hwndDlg, IDC_LOGWIN, EM_FINDTEXTEX, flags, (LPARAM)&ft);
+      ft.chrg.cpMin = (LONG) SendMessage(g_sdata.logwnd, EM_FINDTEXTEX, flags, (LPARAM)&ft);
       if (ft.chrg.cpMin != -1)
-        SendDlgItemMessage(hwndDlg, IDC_LOGWIN, EM_SETSEL, ft.chrgText.cpMin, ft.chrgText.cpMax);
+        SendMessage(g_sdata.logwnd, EM_SETSEL, ft.chrgText.cpMin, ft.chrgText.cpMax);
       else
         MessageBeep(MB_ICONASTERISK);
     }
@@ -906,12 +978,13 @@ static INT_PTR CALLBACK AboutProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM
         else
         {
           dd.AnimPos = finalpos;
-          SetTimer(hwndDlg, ABOUTDLGDATA::TID_HEADER, INFINITE, NULL);
+          KillTimer(hwndDlg, ABOUTDLGDATA::TID_HEADER);
         }
         InvalidateRect(GetDlgItem(hwndDlg, IDC_ABOUTHEADER), NULL, false);
       }
       break;
     case WM_DESTROY:
+      DeleteObject(dd.hHeaderFont);
       DeleteObject(dd.hFont);
       DeleteObject(dd.hBoldFont);
       break;
