@@ -3,7 +3,7 @@
 ** Copyright (C) 2000-2001 Nullsoft, Inc.
 ** Author: Justin Frankel
 ** File: connection.h - JNL TCP connection interface
-** License: see jnetlib.h
+** License: see License.txt
 **
 ** Usage:
 **   1. Create a JNL_Connection object, optionally specifying a JNL_AsyncDNS
@@ -35,7 +35,7 @@
 **            closed, the state will transition to:
 **        JNL_Connection::STATE_CLOSED
 **          - the connection has closed, generally without error. There still
-**            might be data in the receieve buffer, use recv_bytes_available().
+**            might be data in the receive buffer, use recv_bytes_available().
 **   5. Use send() and send_string() to send data. You can use 
 **      send_bytes_in_queue() to see how much has yet to go out, or 
 **      send_bytes_available() to see how much you can write. If you use send()
@@ -50,12 +50,16 @@
 **      make the socket close after sending all the data sent. 
 **  
 **   8. delete ye' ol' object.
+**
+** Unicode support by Jim Park -- 08/24/2007
+**   Keep all the stuff here strictly ANSI.
 */
 
 #ifndef _CONNECTION_H_
 #define _CONNECTION_H_
 
 #include "asyncdns.h"
+#include "netinc.h"
 
 #define JNL_CONNECTION_AUTODNS ((JNL_AsyncDNS*)-1)
 
@@ -77,11 +81,11 @@ class JNL_Connection
     ~JNL_Connection();
 
     void connect(char *hostname, int port);
-    void connect(int sock, struct sockaddr_in *loc=NULL); // used by the listen object, usually not needed by users.
+    void connect(PORTABLE_SOCKET sock, struct sockaddr_in *loc=NULL); // used by the listen object, usually not needed by users.
 
     void run(int max_send_bytes=-1, int max_recv_bytes=-1, int *bytes_sent=NULL, int *bytes_rcvd=NULL);
     int  get_state() { return m_state; }
-    char *get_errstr() { return m_errorstr; }
+    const char *get_errstr() { return m_errorstr; }
 
     void close(int quick=0);
     void flush_send(void) { m_send_len=m_send_pos=0; }
@@ -107,7 +111,7 @@ class JNL_Connection
     short get_remote_port(void) { return m_remote_port; } // this returns the remote port of connection
   
   protected:
-    int  m_socket;
+    PORTABLE_SOCKET m_socket;
     short m_remote_port;
     char *m_recv_buffer;
     char *m_send_buffer;
@@ -126,7 +130,7 @@ class JNL_Connection
     int m_dns_owned;
 
     state m_state;
-    char *m_errorstr;
+    const char *m_errorstr;
 
     int getbfromrecv(int pos, int remove); // used by recv_line*
 
