@@ -20,8 +20,8 @@ int main(int argc, char* argv[])
   runner.setOutputter( new CppUnit::CompilerOutputter( &runner.result(),
                                                        std::cerr ) );
   // Run the tests.
-  bool wasSucessful = runner.run();
+  bool wasSuccessful = runner.run();
 
   // Return error code 1 if the one of test failed.
-  return wasSucessful ? 0 : 1;
+  return wasSuccessful ? 0 : 1;
 }
