@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2019 Nullsoft and Contributors
+ * Copyright (C) 1999-2023 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -61,7 +61,7 @@
 // 5000..5999 Important generic warnings
 // 6000..6999 Script warnings
 // 7000..7499 Recovered from bad input etc. warnings
-// 7500..7999 Discouraged usage warnings (allocated top to bottom to reserve as much space as possible for more bad input codes)
+// 7500..7999 Discouraged usage warnings (allocated high to low to reserve as much space as possible for more bad input codes)
 // 8000..8999 Generic warnings
 // 9000..9999 Breaking our and/or MS guidelines warnings
 typedef enum {
@@ -71,6 +71,7 @@ typedef enum {
   //DE_PP_VERBOSE_BAD_LEVEL = 4000?,
   //DW_STALE_TEMP = 5020?, TODO: There is currently no way to disable this
   DW_PACKHDR_RETNONZERO = 5021,
+  DW_BADFORMAT_EXTERNAL_FILE = 5040,
   DW_UNSUPP_STORE_FILE_ATT = 5050,
   //DW_CMDLINE_UNSUPP = 5200?,
   //DW_CMDLINE_UNSUPP_WIN = 5201?,
@@ -107,6 +108,7 @@ typedef enum {
   DW_LICENSE_EMPTY = 7050,
   DW_ATTRIBUTE_OVERLONGSTRING = 7060,
   DW_PARSE_BADNUMBER = 7070,
+  DW_PARSE_NUMBEROUTOFSPEC = 7071,
   DW_PARSE_LNK_HK = 7075,
   DW_GENERIC_DEPRECATED = 7998,
   DW_PARSE_REGPATHPREFIX = 7999,
@@ -143,11 +145,12 @@ namespace MakensisAPI {
   extern const TCHAR* SigintEventNameFmt;
   extern const TCHAR* SigintEventNameLegacy;
 
-  enum notify_e {
-    NOTIFY_SCRIPT, // main nsi file(s)
+  enum datatransfer_e {
+    NOTIFY_SCRIPT, // Compiler -> Host: main nsi file(s)
     NOTIFY_WARNING,
     NOTIFY_ERROR,
-    NOTIFY_OUTPUT // generated .exe file
+    NOTIFY_OUTPUT, // Compiler -> Host: Generated .exe file
+    PROMPT_FILEPATH // [0x03006000] Compiler -> Host -> Compiler
   };
 #ifdef _WIN32
   enum sndmsg_e {
@@ -156,8 +159,14 @@ namespace MakensisAPI {
 #endif
   enum QUERYHOST_e {
     QH_OUTPUTCHARSET = 1, // [0x03000000] return (wincodepage+1) or 0 for default (This encoding is used by stdout, stderr and the notify messages)
-    QH_ENABLESTDERR // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_ENABLESTDERR, // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_SUPPORTEDVERSION, // [0x03006000] The host must return new highest makensis compiler version it supports
   };
+  typedef struct {
+    unsigned char Platform; // Bitness OR'ed with sizeof(TCHAR) of the compiler
+    unsigned char Reserved;
+    TCHAR Path[1];
+  } PROMPT_FILEPATH_DATA;
 }
 
 #define PAGE_CUSTOM 0
@@ -244,6 +253,7 @@ class CEXEBuild {
     unsigned int get_header_size() const { return (unsigned int)sizeof(header) + (is_target_64bit() ? (4 * BLOCKS_NUM) : 0); }
 
     void set_default_output_filename(const tstring& filename);
+    const TCHAR* get_output_filename() const { return build_output_filename; }
 
     // process a script (you can process as many scripts as you want,
     // it is as if they are concatenated)
@@ -273,7 +283,14 @@ class CEXEBuild {
     NStreamLineReader* curlinereader;
 
     HWND notify_hwnd;
-    void notify(MakensisAPI::notify_e code, const TCHAR *data) const;
+    void notify(MakensisAPI::datatransfer_e code, const TCHAR *data) const;
+    #ifdef _WIN32
+    typedef bool (CALLBACK*HOSTAPIREQUESTDATAPROC)(void*cookie, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
+    #else
+    typedef bool (*HOSTAPIREQUESTDATAPROC)(void*cookie);
+    #endif
+    bool hostapi_request_data(MakensisAPI::datatransfer_e operation, UINT minver, HOSTAPIREQUESTDATAPROC proc, void*cookie, const void* input, size_t inputsize) const;
+    bool prompt_for_output_path(TCHAR*path, UINT pathcap) const;
 
   private:
     int check_write_output_errors() const;
@@ -315,6 +332,7 @@ class CEXEBuild {
     int parseScript();
     int includeScript(const TCHAR *f, NStreamEncoding&enc);
     TCHAR* GetMacro(const TCHAR *macroname, TCHAR**macroend = 0);
+    TCHAR* GetMacro(size_t idx);
     bool MacroExists(const TCHAR *macroname) { return !!GetMacro(macroname); }
     LANGID ParseLangIdParameter(const LineParser&line, int token);
     int LoadLicenseFile(const TCHAR *file, TCHAR** pdata, const TCHAR *cmdname, WORD AnsiCP);
@@ -327,6 +345,7 @@ class CEXEBuild {
     int doParse(const TCHAR *str);
     int doCommand(int which_token, LineParser &line);
     TCHAR m_templinebuf[MAX_LINELENGTH]; // Buffer used by parseScript() & doCommand(), not recursion safe!
+    int add_flag_instruction_entry(int which_token, int opcode, LineParser &line, int offset, int data = 0);
 
     int do_add_file(const TCHAR *lgss, int attrib, int recurse, int *total_files, const TCHAR 
       *name_override=0, int generatecode=1, int *data_handle=0, 
@@ -357,6 +376,8 @@ class CEXEBuild {
     bool inside_comment;
     int multiple_entries_instruction;  // 1 (true) or 0 (false)
 
+    int pp_boolifyexpression(LineParser&line, int &result, bool allow_logicneg = true, int ignore_last_tokens = 0);
+    int pp_assert(LineParser&line);
     int pp_macro(LineParser&line);
     int pp_macroundef(LineParser&line);
     int pp_insertmacro(LineParser&line);
@@ -372,7 +393,7 @@ class CEXEBuild {
     int pp_define(LineParser&line);
     int pp_undef(LineParser&line);
     int pp_packhdr(LineParser&line);
-    int pp_finalize(LineParser&line);
+    int pp_finalize(int which_token, LineParser&line);
     int pp_execute(int which_token, LineParser&line);
     int pp_addincludedir(LineParser&line);
     int pp_include(LineParser&line);
@@ -552,6 +573,7 @@ class CEXEBuild {
     int DeclaredUserVar(const TCHAR *VarName);
     void VerifyDeclaredUserVarRefs(UserVarsStringList *pVarsStringList);
     bool IsIntOrUserVar(const LineParser &line, int token) const;
+    static bool IsVarPrefix(const TCHAR*s) { return *s++ == _T('$') && *s > ' '; }
 
     ConstantsStringList m_ShellConstants;
 
@@ -587,7 +609,7 @@ class CEXEBuild {
       TCHAR cmd[1];
       void delete_all();
       static postbuild_cmd* make(const TCHAR *cmdstr, int cmpop, int cmpval);
-    } *postbuild_cmds;
+    } *postbuild_cmds, *postubuild_cmds;
     int run_postbuild_cmds(const postbuild_cmd *cmds, const TCHAR *templatearg_pc1, const TCHAR* commandname);
     int check_external_exitcode(int exitcode, int op, int val);
 
@@ -598,7 +620,7 @@ class CEXEBuild {
 
     header build_header, build_uninst, *cur_header;
     int uninstall_mode; // Are we in uninstall mode?  Acts like a bool.
-    int uninstall_size,uninstall_size_full;
+    UINT32 uninstall_size,uninstall_size_full;
     int uninstaller_writes_used;
 
     TCHAR build_output_filename[1024];
