@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2019 Nullsoft, Jeff Doozan and Contributors
+ * Copyright (C) 1999-2023 Nullsoft, Jeff Doozan and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -672,9 +672,14 @@ skipPage:
   }
   if (uMsg == WM_SIZE) {
     ShowWindow(m_bgwnd, wParam == SIZE_MINIMIZED ? SW_HIDE : SW_SHOW);
+#else //! NSIS_SUPPORT_BGBG
+  if (uMsg == WM_SIZE) {
+#endif //~ NSIS_SUPPORT_BGBG
+    if (wParam == SIZE_MAXIMIZED) {
+      DWORD style = (DWORD) GetWindowLongPtr(hwndDlg, GWL_STYLE), mask = WS_MAXIMIZEBOX|WS_MAXIMIZE|WS_MINIMIZE;
+      if ((style & mask) == WS_MAXIMIZE) ShowWindow(hwndDlg, SW_SHOWNOACTIVATE); // Disallow STARTF_USESHOWWINDOW+SW_MAXIMIZE unless someone does ${NSD_AddStyle} $hWndParent ${WS_MAXIMIZEBOX}
+    }
   }
-#endif //NSIS_SUPPORT_BGBG
-
   if (uMsg == WM_NOTIFY_CUSTOM_READY) {
     DestroyWindow(m_curwnd);
     m_curwnd = (HWND)wParam;
@@ -1236,18 +1241,19 @@ static void FORCE_INLINE NSISCALL RefreshComponents(HWND hwTree, HTREEITEM *item
     TreeView_SetItem(hwTree, &item);
   }
 
-  // workaround for bug #1397031
+  // workaround for bug #1397031 A.K.A #434
   //
-  // windows 95 doesn't erase the background of the state image
-  // before it draws a new one. because of this parts of the old
+  // Windows 95 & NT4 doesn't erase the background of the state image
+  // before it draws a new one. Because of this parts of the old
   // state image will show where the new state image is masked.
   //
-  // to solve this, the following line forces the background to
-  // be erased. sadly, this redraws the entire control. it might
+  // To solve this, the following line forces the background to
+  // be erased. sadly, this redraws the entire control. It might
   // be a good idea to figure out where the state images are and
   // redraw only those.
 
-  InvalidateRect(hwTree, NULL, TRUE);
+  if (IsWin95NT4()) // Checking for < IE4 is probably better but more work
+    InvalidateRect(hwTree, NULL, TRUE);
 }
 
 int NSISCALL TreeGetSelectedSection(HWND tree, BOOL mouse)
