@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2019 Nullsoft and Contributors
+ * Copyright (C) 1999-2023 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,8 +45,9 @@ typedef struct _stack_t {
 static stack_t *g_st;
 #endif
 
-exec_flags_t g_exec_flags;
+
 exec_flags_t g_exec_flags_last_used;
+execflags_and_osinfo g_execflags_and_osinfo;
 
 extra_parameters plugin_extra_parameters = {
   &g_exec_flags,
@@ -176,33 +177,42 @@ static HKEY NSISCALL RegCreateScriptKey(int RootKey, LPCTSTR SubKey, REGSAM RS)
 // RegDeleteKeyEx on 32-bit Windows accepts but ignores the KEY_WOW64_xxKEY flags and always uses the 
 // one and only native key. Our RegKeyOpen will intentionally fail if a incompatible WoW64 flag is used.
 #define DRTF_ONLYIFNOSUBKEYS DELREGKEY_ONLYIFNOSUBKEYS
+#define DRTF_ONLYIFNOVALUES  DELREGKEY_ONLYIFNOVALUES
 static LONG NSISCALL DeleteRegTree(HKEY hThisKey, LPCTSTR SubKey, REGSAM SamviewAndFlags)
 {
   HKEY hKey;
-  UINT onlyifempty = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnosubkeys = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnovalues  = SamviewAndFlags & DRTF_ONLYIFNOVALUES, valuesexistcheckinsubkeys = TRUE;
   REGSAM samview = SamviewAndFlags & (KEY_WOW64_32KEY|KEY_WOW64_64KEY);
-  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|samview, &hKey);
+  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|KEY_QUERY_VALUE|samview, &hKey);
   if (retval == ERROR_SUCCESS)
   {
     TCHAR child[MAX_PATH+1]; // NB - don't change this to static (recursive function)
+    if (onlyifnovalues)
+    {
+      DWORD cchName = 0;
+      retval = RegEnumValue(hKey, 0, child, &cchName, NULL, NULL, NULL, NULL);
+      if (retval != ERROR_NO_MORE_ITEMS) goto notempty;
+      if (!valuesexistcheckinsubkeys) SamviewAndFlags &= ~DRTF_ONLYIFNOVALUES;
+    }
     while (RegEnumKey(hKey, 0, child, COUNTOF(child)) == ERROR_SUCCESS)
     {
-      if (onlyifempty) return (RegCloseKey(hKey), ERROR_CAN_NOT_COMPLETE);
+      if (onlyifnosubkeys) notempty: return (RegCloseKey(hKey), ERROR_CAN_NOT_COMPLETE);
       if ((retval = DeleteRegTree(hKey, child, SamviewAndFlags)) != ERROR_SUCCESS) break;
     }
     RegCloseKey(hKey);
     {
-      typedef LONG (WINAPI * RegDeleteKeyExPtr)(HKEY, LPCTSTR, REGSAM, DWORD);
-      RegDeleteKeyExPtr RDKE = (RegDeleteKeyExPtr)
-      #ifdef _WIN64
-        RegDeleteKeyEx;
-      #else
+      typedef LONG (WINAPI * RegDeleteKeyExType)(HKEY, LPCTSTR, REGSAM, DWORD);
+      RegDeleteKeyExType RDKE = (RegDeleteKeyExType)
+      #if !defined(_WIN64) || defined(_M_IA64)
         myGetProcAddress(MGA_RegDeleteKeyEx);
-      #endif
-      if (RDKE)
-        retval = RDKE(hThisKey, SubKey, samview, 0);
-      else
+      if (!RDKE)
         retval = RegDeleteKey(hThisKey, SubKey);
+      else
+      #else
+        RegDeleteKeyEx;
+      #endif
+        retval = RDKE(hThisKey, SubKey, samview, 0);
     }
   }
   return retval;
@@ -261,7 +271,14 @@ static int NSISCALL ExecuteEntry(entry *entry_)
   //var4 = g_usrvars[parm4];
   //var5 = g_usrvars[parm5];
 
+#if __GNUC__ >= 12
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdangling-pointer"
+#endif
   g_parms = lent.offsets;
+#if __GNUC__ >= 12
+#pragma GCC diagnostic pop
+#endif
 
   switch (which)
   {
@@ -302,9 +319,10 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     break;
 #endif//NSIS_CONFIG_VISIBLE_SUPPORT
     case EW_SETFLAG:
-      if (!parm2)
+      if (parm2 <= 0)
       {
-        FIELDN(g_exec_flags_last_used,parm0)=FIELDN(g_exec_flags,parm0);
+        if (parm2 == 0)
+          FIELDN(g_exec_flags_last_used,parm0)=FIELDN(g_exec_flags,parm0);
         FIELDN(g_exec_flags,parm0)=GetIntFromParm(1);
         log_printf3(_T("SetFlag: %d=%d"),parm0,FIELDN(g_exec_flags,parm0));
       }
@@ -834,14 +852,15 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     case EW_LOADANDSETIMAGE:
     {
       RECT r;
-      HANDLE hImage;
-      HWND hCtl=(parm2 & LASIF_HWND) ? GetHwndFromParm(1) : GetDlgItem(g_hwnd, parm1);
-      UINT it=parm2 & LASIM_IMAGE, exeres=parm2 & LASIF_EXERES, fitw=(UINT)parm2 >> LASIS_FITCTLW, fith=(parm2 & LASIF_FITCTLH) != 0;
-      LPCTSTR imgname = (parm2 & LASIF_STRID) ? GetStringFromParm(0x00) : MAKEINTRESOURCE(parm0);
+      HANDLE hNewImage, hPrevImage;
+      HWND hCtl=(parm3 & LASIF_HWND) ? GetHwndFromParm(2) : GetDlgItem(g_hwnd, parm2);
+      UINT it=parm3 & LASIM_IMAGE, exeres=parm3 & LASIF_EXERES, fitw=(UINT)parm3 >> LASIS_FITCTLW, fith=(parm3 & LASIF_FITCTLH) != 0;
+      LPCTSTR imgid = (parm3 & LASIF_STRID) ? GetStringFromParm(0x11) : MAKEINTRESOURCE(parm1);
       GetClientRect(hCtl, &r);
-      hImage=LoadImage(exeres ? g_hInstance : NULL, imgname, it, fitw*r.right, fith*r.bottom, parm2 & LASIM_LR);
-      hImage=(HANDLE)SendMessage(hCtl, STM_SETIMAGE, it, (LPARAM)hImage);
-      if (hImage && IMAGE_BITMAP == it) DeleteObject(hImage); // Delete the old image
+      hNewImage=LoadImage(exeres ? g_hInstance : NULL, imgid, it, fitw*r.right, fith*r.bottom, parm3 & LASIM_LR);
+      hPrevImage=(HANDLE)SendMessage(hCtl, STM_SETIMAGE, it, (LPARAM)hNewImage);
+      if (hPrevImage && IMAGE_BITMAP == it) DeleteObject(hPrevImage); // Delete the old bitmap
+      if (parm0 >= 0) iptrtostr(var0, (INT_PTR)hNewImage); // Optional output handle
     }
     break;
     case EW_CREATEFONT:
@@ -976,8 +995,8 @@ static int NSISCALL ExecuteEntry(entry *entry_)
             if ( ((BOOL(WINAPI*)(LPCTSTR,DWORD,DWORD,LPVOID))gfvi)(buf1,0,s1,b1)
               && ((BOOL(WINAPI*)(LPCVOID,LPCTSTR,LPVOID*,UINT*))vqv)(b1,_T("\\"),(void*)&pvsf1,&uLen) )
             {
-              myitoa(highout,pvsf1->dwFileVersionMS);
-              myitoa(lowout,pvsf1->dwFileVersionLS);
+              myitoa(highout,(&pvsf1->dwFileVersionMS)[parm3]);
+              myitoa(lowout,(&pvsf1->dwFileVersionLS)[parm3]);
 
               exec_error--;
             }
@@ -1390,7 +1409,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         else if (which==EW_FPUTS)
         {
           GetStringFromParm(0x21); // load string in buf2, convert it to ANSI in buf1
-          WideCharToMultiByte(CP_ACP, 0, buf2, -1, (LPSTR) buf1, NSIS_MAX_STRLEN, NULL, NULL);
+          StrWideToACP(buf2, (LPSTR) buf1, NSIS_MAX_STRLEN);
           l=lstrlenA((LPCSTR)buf1);
         }
 #endif
@@ -1553,36 +1572,41 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         hFile=myOpenFile(buf1,GENERIC_WRITE,CREATE_ALWAYS);
         if (hFile != INVALID_HANDLE_VALUE)
         {
-          unsigned char *filebuf;
-          int filehdrsize = g_filehdrsize;
-          filebuf=(unsigned char *)GlobalAlloc(GPTR,filehdrsize);
-          if (filebuf)
+          int dboffset = parm1;
+          if (parm2)
           {
-            SetSelfFilePointer(0);
-            ReadSelfFile((char*)filebuf,filehdrsize);
+            unsigned char *filebuf;
+            int filehdrsize = g_filehdrsize;
+            filebuf=(unsigned char *)GlobalAlloc(GPTR,filehdrsize);
+            if (filebuf)
             {
-              // parm1 = uninstdata_offset
-              // parm2 = m_unicon_size
-              unsigned char* seeker;
-              unsigned char* unicon_data = seeker = (unsigned char*)GlobalAlloc(GPTR,parm2);
-              if (unicon_data) {
-                GetCompressedDataFromDataBlockToMemory(parm1,unicon_data,parm2);
-                while (*seeker) {
-                  struct icondata {
-                    DWORD dwSize;
-                    DWORD dwOffset;
-                  } id = *(struct icondata *) seeker;
-                  seeker += sizeof(struct icondata);
-                  mini_memcpy(filebuf+id.dwOffset, seeker, id.dwSize);
-                  seeker += id.dwSize;
+              SetSelfFilePointer(0);
+              ReadSelfFile((char*)filebuf,filehdrsize);
+              {
+                // parm1 = uninstdata_offset
+                // parm2 = m_unicon_size
+                unsigned char* seeker;
+                unsigned char* unicon_data = seeker = (unsigned char*)GlobalAlloc(GPTR,parm2);
+                if (unicon_data) {
+                  GetCompressedDataFromDataBlockToMemory(parm1,unicon_data,parm2);
+                  while (*seeker) {
+                    struct icondata {
+                      DWORD dwSize;
+                      DWORD dwOffset;
+                    } id = *(struct icondata *) seeker;
+                    seeker += sizeof(struct icondata);
+                    mini_memcpy(filebuf+id.dwOffset, seeker, id.dwSize);
+                    seeker += id.dwSize;
+                  }
+                  GlobalFree(unicon_data);
                 }
-                GlobalFree(unicon_data);
               }
+              myWriteFile(hFile,(char*)filebuf,filehdrsize);
+              GlobalFree(filebuf);
+              dboffset = -1;
             }
-            myWriteFile(hFile,(char*)filebuf,filehdrsize);
-            GlobalFree(filebuf);
-            ret=GetCompressedDataFromDataBlock(-1,hFile);
           }
+          ret=GetCompressedDataFromDataBlock(dboffset, hFile);
           CloseHandle(hFile);
         }
         log_printf3(_T("created uninstaller: %d, \"%s\""),ret,buf1);
@@ -1704,6 +1728,42 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     break;
 #endif//NSIS_CONFIG_COMPONENTPAGE
 
+    case EW_GETOSINFO:
+    {
+      switch(parm3)
+      {
+#ifdef NSIS_SUPPORT_FNUTIL
+        case GETOSINFO_KNOWNFOLDER:
+        {
+          TCHAR *outstr = var1;
+          IID kfid;
+          HRESULT(WINAPI*SHGKFP)(REFIID,DWORD,HANDLE,PWSTR*) = (HRESULT(WINAPI*)(REFIID,DWORD,HANDLE,PWSTR*)) myGetProcAddress(MGA_SHGetKnownFolderPath);
+          TCHAR *buf2 = GetStringFromParm(0x22), succ = FALSE;
+          if (SHGKFP && SUCCEEDED(ComIIDFromString(buf2, &kfid)))
+          {
+            PWSTR path;
+            HRESULT hr = SHGKFP(&kfid, parm3, NULL, &path);
+            if (SUCCEEDED(hr))
+              strcpyWideToT(outstr, path), CoTaskMemFree(path), ++succ;
+          }
+          if (!succ)
+            exec_error++, *outstr = _T('\0');
+        }
+        break;
+#endif
+        case GETOSINFO_READMEMORY:
+        {
+          TCHAR *outstr = var1;
+          SIZE_T addr = GetIntPtrFromParm(2), spec = GetIntPtrFromParm(4), value = 0;
+          UINT cb = LOBYTE(spec), offset = (UINT)(spec) >> 24;
+          if (addr == ABI_OSINFOADDRESS) addr = (SIZE_T) (&g_execflags_and_osinfo);
+          mini_memcpy(&value, ((char*) addr) + offset, cb);
+          iptrtostr(outstr, value);
+        }
+        break;
+      }
+    }
+    break;
 #ifdef NSIS_LOCKWINDOW_SUPPORT
     case EW_LOCKWINDOW:
     {
