@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2019 Nullsoft and Contributors
+ * Copyright (C) 1999-2023 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -61,12 +61,15 @@ extern FILE *g_output, *g_errout;
 
 
 #ifdef _WIN32
-static char* CreateMappedFileView(LPCTSTR Path, DWORD FAccess, DWORD FShare, DWORD FMode, DWORD PProtect, DWORD MAccess)
+static char* CreateMappedFileView(LPCTSTR Path, DWORD FAccess, DWORD FShare, DWORD FMode, DWORD PProtect, DWORD MAccess, size_t &FSize)
 {
-  char *pView = NULL, restoreGLE = false;
+  char *pView = NULL, restoreGLE = false, validSize;
   HANDLE hFile = CreateFile(Path, FAccess, FShare, NULL, FMode, 0, NULL);
   if (hFile == INVALID_HANDLE_VALUE) return pView;
-  HANDLE hMap = CreateFileMapping(hFile, NULL, PProtect, 0, 0, NULL);
+  ULARGE_INTEGER fs;
+  validSize = GetFileSize64(hFile, fs) && (sizeof(size_t) >= 8 || !fs.HighPart);
+  FSize = sizeof(size_t) >= 8 ? (size_t) fs.QuadPart : fs.LowPart;
+  HANDLE hMap = validSize ? CreateFileMapping(hFile, NULL, PProtect, 0, 0, NULL) : INVALID_HANDLE_VALUE;
   if (hMap != INVALID_HANDLE_VALUE)
   {
     CloseHandle(hFile);
@@ -146,7 +149,7 @@ int update_bitmap(CResourceEditor* re, WORD id, const TCHAR* filename, int width
   signed char hdr[14+124], retval = -2;
   size_t size = fread(hdr, 1, sizeof(hdr), f);
   GENERICIMAGEINFO info;
-  if (IsBMPFile(hdr, size, &info) && 0 == fseek(f, 0, SEEK_SET) && !info.IsTopDownBitmap())
+  if (IsBMPFile(hdr, size, &info) && 0 == fseek(f, 0, SEEK_SET) && LoadImageCanLoadFileFromResource(hdr, size))
   {
     if ((width && width != (int) info.Width) || (height && height != (int) info.Height))
       retval = -3;
@@ -196,9 +199,9 @@ TCHAR *CharPrev(const TCHAR *s, const TCHAR *p) {
 }
 
 char *CharNextA(const char *s) {
-  int l = 0;
+  int l = 0, mbl;
   if (s && *s)
-    l = max(1, mblen(s, MB_CUR_MAX));
+    mbl = mblen(s, MB_CUR_MAX), l = max(1, mbl);
   return const_cast<char*>(s + l);
 }
 
@@ -637,28 +640,24 @@ FILE* my_fopen(const TCHAR *path, const char *mode)
   return f;
 }
 
+#include <sys/types.h>
+#include <sys/stat.h>
 #if (defined(_MSC_VER) && (_MSC_VER >= 1200)) || defined(__MINGW32__)
 #include <io.h>
-static UINT64 get_file_size64(FILE *f)
+UINT64 get_file_size64(FILE *f)
 {
   INT64 s = _filelengthi64(_fileno(f)); // Could also use _get_osfhandle+GetFileSize64
   return (INT64) -1L != s ? s : invalid_file_size64;
 }
-#endif
-
-#include <sys/types.h>
-#include <sys/stat.h>
-UINT32 get_file_size32(FILE *f)
+#else
+UINT64 get_file_size64(FILE *f)
 {
-  UINT32 result = invalid_file_size32;
-#if (defined(_MSC_VER) && (_MSC_VER >= 1200)) || defined(__MINGW32__)
-  UINT64 size64 = get_file_size64(f);
-  if (invalid_file_size64 != size64 && size64 <= 0xffffffffUL)
-    result = (UINT32) size64;
-#elif _XOPEN_SOURCE >= 500 || _POSIX_C_SOURCE >= 200112L
+  UINT64 result = invalid_file_size64;
+  // 32bit plaforms require _FILE_OFFSET_BITS = 64 to correctly return the size
+#if _XOPEN_SOURCE >= 500 || _POSIX_C_SOURCE >= 200112L
   struct stat st;
-  if (0 == fstat(fileno(f), &st) && st.st_size <= (sizeof(st.st_size) >= 8 ? (off_t)0xffffffffUL : LONG_MAX))
-    result = (UINT32) st.st_size;
+  if (0 == fstat(fileno(f), &st) && st.st_size <= (sizeof(st.st_size) >= 8 ? (off_t)0x7fffffffffffffffLL : LONG_MAX))
+    result = (UINT64) st.st_size;
 #else
   long cb, restoreseek = true;
   fpos_t orgpos;
@@ -670,6 +669,41 @@ UINT32 get_file_size32(FILE *f)
 #endif
   return result;
 }
+#endif
+
+UINT32 get_file_size32(FILE *f)
+{
+  UINT32 result = invalid_file_size32;
+#if _XOPEN_SOURCE >= 500 || _POSIX_C_SOURCE >= 200112L
+  struct stat st;
+  if (0 == fstat(fileno(f), &st) && st.st_size <= (sizeof(st.st_size) >= 8 ? (off_t)0xffffffffUL : LONG_MAX))
+    result = (UINT32) st.st_size;
+#else
+  UINT64 size64 = get_file_size64(f);
+  if (size64 <= 0xffffffffUL && invalid_file_size64 != size64)
+    result = (UINT32) size64;
+#endif
+  return result;
+}
+
+UINT64 Platform_GetMaxFileSize()
+{
+#ifdef _WIN32
+  return ~(UINT64)0;
+#elif _XOPEN_SOURCE >= 500 || _POSIX_C_SOURCE >= 200112L
+  struct stat st;
+  if (sizeof(st.st_size) >= 8)
+  {
+#ifdef LLONG_MAX
+    return (UINT64)(off_t)LLONG_MAX;
+#endif
+    const UINT64 sixthree = (~(UINT64)0) >> 1;
+    return (off_t)sixthree > 0 ? sixthree : (UINT32)(off_t)0xffffffffUL;
+  }
+  return (UINT64)(off_t)LONG_MAX;
+#endif
+  return (UINT)INT_MAX;
+}
 
 BYTE* alloc_and_read_file(FILE *f, unsigned long &size)
 {
@@ -702,18 +736,27 @@ void close_file_view(FILEVIEW&mmfv)
 #ifdef _WIN32
   if (mmfv.base) UnmapViewOfFile(mmfv.base);
 #else
-  if (mmfv.base) munmap(mmfv.base, mmfv.internal);
+  if (mmfv.base) munmap(mmfv.base, mmfv.size);
 #endif
 }
 char* create_file_view_readonly(const TCHAR *filepath, FILEVIEW&mmfv)
 {
 #ifdef _WIN32
-  return mmfv.base = CreateMappedFileView(filepath, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, PAGE_READONLY, FILE_MAP_READ);
+  return mmfv.base = CreateMappedFileView(filepath, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, PAGE_READONLY, FILE_MAP_READ, mmfv.size);
 #else
-  return mmfv.base = CreateMappedFileView(filepath, "rb", PROT_READ, MAP_SHARED, mmfv.internal);
+  return mmfv.base = CreateMappedFileView(filepath, "rb", PROT_READ, MAP_SHARED, mmfv.size);
 #endif
 }
 
+size_t write_octets_to_file(const TCHAR *filename, const void *data, size_t cb)
+{
+  FILE *hfile = FOPEN(filename, ("wb"));
+  if (!hfile) return 0;
+  size_t ret = fwrite(data, 1, cb, hfile);
+  fclose(hfile);
+  return ret;
+}
+
 TCHAR* create_tempfile_path()
 {
   TCHAR *tfpath = NULL;
@@ -810,34 +853,30 @@ tstring get_executable_path(const TCHAR* argv0) {
   assert(rc == 0);
   return tstring(CtoTString(temp_buf));
 #else /* Linux/BSD/POSIX/etc */
-  const TCHAR *envpath = _tgetenv(_T("_"));
-  if (envpath)
-    return get_full_path(envpath);
-  else {
-    char *path = NULL, *pathtmp;
-    size_t len = 100;
-    int nchars;
-    while(1){
-      pathtmp = (char*)realloc(path,len+1);
-      if (pathtmp == NULL) {
-        free(path);
-        return get_full_path(argv0);
-      }
-      path = pathtmp;
-      nchars = readlink("/proc/self/exe", path, len);
-      if (nchars == -1) {
-        free(path);
-        return get_full_path(argv0);
-      }
-      if (nchars < (int) len) {
-        path[nchars] = '\0';
-        tstring result;
-        result = CtoTString(path);
-        free(path);
-        return result;
-      }
-      len *= 2;
+  char *path = NULL, *pathtmp;
+  size_t len = 100;
+  int nchars;
+  while(1){
+    pathtmp = (char*)realloc(path,len+1);
+    if (!pathtmp) {
+basic:
+      free(path);
+      // Note: Not using _tgetenv(_T("_")) here because it is not always set correctly. !makensis instruction in SVN r7372 will execute "/usr/bin/scons"!
+      return get_full_path(argv0);
     }
+    path = pathtmp;
+    nchars = readlink("/proc/self/exe", path, len);
+    if (nchars == -1) {
+      goto basic;
+    }
+    if (nchars < (int) len) {
+      path[nchars] = '\0';
+      tstring result;
+      result = CtoTString(path);
+      free(path);
+      return result;
+    }
+    len *= 2;
   }
 #endif
 }
